@@ -20,6 +20,7 @@ from __future__ import (absolute_import, division, print_function)
 # Python
 import collections
 import contextlib
+import datetime
 import sys
 import uuid
 from copy import copy
@@ -36,6 +37,10 @@ from .minimal import CallbackModule as MinimalCallbackModule
 CENSORED = "the output has been hidden due to the fact that 'no_log: true' was specified for this result"  # noqa
 
 
+def current_time():
+    return datetime.datetime.utcnow()
+
+
 class BaseCallbackModule(CallbackBase):
     '''
     Callback module for logging ansible/ansible-playbook events.
@@ -62,6 +67,7 @@ class BaseCallbackModule(CallbackBase):
 
     def __init__(self):
         super(BaseCallbackModule, self).__init__()
+        self._host_start = {}
         self.task_uuids = set()
         self.duplicate_task_counts = collections.defaultdict(lambda: 1)
 
@@ -169,7 +175,7 @@ class BaseCallbackModule(CallbackBase):
 
     def v2_playbook_on_vars_prompt(self, varname, private=True, prompt=None,
                                    encrypt=None, confirm=False, salt_size=None,
-                                   salt=None, default=None):
+                                   salt=None, default=None, unsafe=None):
         event_data = dict(
             varname=varname,
             private=private,
@@ -179,6 +185,7 @@ class BaseCallbackModule(CallbackBase):
             salt_size=salt_size,
             salt=salt,
             default=default,
+            unsafe=unsafe,
         )
         with self.capture_event_data('playbook_on_vars_prompt', **event_data):
             super(BaseCallbackModule, self).v2_playbook_on_vars_prompt(
@@ -352,11 +359,15 @@ class BaseCallbackModule(CallbackBase):
         if result._task.action in ('setup', 'gather_facts'):
             result._result.get('ansible_facts', {}).pop('ansible_env', None)
 
+        end_time = current_time()
         event_data = dict(
             host=result._host.get_name(),
             remote_addr=result._host.address,
             task=result._task,
             res=result._result,
+            start=self._host_start[result._host.get_name()],
+            end=end_time,
+            duration=(end_time - self._host_start[result._host.get_name()]).total_seconds(),
             event_loop=self._get_event_loop(result._task),
         )
         with self.capture_event_data('runner_on_ok', **event_data):
@@ -364,11 +375,15 @@ class BaseCallbackModule(CallbackBase):
 
     def v2_runner_on_failed(self, result, ignore_errors=False):
         # FIXME: Add verbosity for exception/results output.
+        end_time = current_time()
         event_data = dict(
             host=result._host.get_name(),
             remote_addr=result._host.address,
             res=result._result,
             task=result._task,
+            start=self._host_start[result._host.get_name()],
+            end=end_time,
+            duration=(end_time - self._host_start[result._host.get_name()]).total_seconds(),
             ignore_errors=ignore_errors,
             event_loop=self._get_event_loop(result._task),
         )
@@ -376,20 +391,28 @@ class BaseCallbackModule(CallbackBase):
             super(BaseCallbackModule, self).v2_runner_on_failed(result, ignore_errors)
 
     def v2_runner_on_skipped(self, result):
+        end_time = current_time()
         event_data = dict(
             host=result._host.get_name(),
             remote_addr=result._host.address,
             task=result._task,
+            start=self._host_start[result._host.get_name()],
+            end=end_time,
+            duration=(end_time - self._host_start[result._host.get_name()]).total_seconds(),
             event_loop=self._get_event_loop(result._task),
         )
         with self.capture_event_data('runner_on_skipped', **event_data):
             super(BaseCallbackModule, self).v2_runner_on_skipped(result)
 
     def v2_runner_on_unreachable(self, result):
+        end_time = current_time()
         event_data = dict(
             host=result._host.get_name(),
             remote_addr=result._host.address,
             task=result._task,
+            start=self._host_start[result._host.get_name()],
+            end=end_time,
+            duration=(end_time - self._host_start[result._host.get_name()]).total_seconds(),
             res=result._result,
         )
         with self.capture_event_data('runner_on_unreachable', **event_data):
@@ -497,6 +520,7 @@ class BaseCallbackModule(CallbackBase):
             host=host.get_name(),
             task=task
         )
+        self._host_start[host.get_name()] = current_time()
         with self.capture_event_data('runner_on_start', **event_data):
             super(BaseCallbackModule, self).v2_runner_on_start(host, task)
 
